@@ -18,7 +18,7 @@ BEGIN
     $extra = 1
         if $st ;
 
-    plan(tests => 597 + $extra) ;
+    plan(tests => 641 + $extra) ;
 }
 
 sub myGZreadFile
@@ -161,6 +161,7 @@ sub run
 
 
             my $lex = new LexFile my $name ;
+            #my $name = "/tmp/try.lzf";
 
             my $hello = <<EOM ;
 hello world
@@ -544,6 +545,37 @@ EOM
 
         }
 
+        {
+            # embed a compressed file in another buffer
+            #================================
+
+
+            my $hello = <<EOM ;
+hello world
+this is a test
+EOM
+
+            my $trailer = "trailer data" ;
+
+            my $compressed ;
+
+            {
+              ok my $x = new $CompressClass(\$compressed);
+
+              ok $x->write($hello) ;
+              ok $x->close ;
+              $compressed .= $trailer ;
+            }
+
+            my $uncomp;
+            ok my $x = new $UncompressClass(\$compressed, Append => 1)  ;
+            1 while $x->read($uncomp) > 0 ;
+
+            ok $uncomp eq $hello ;
+            is $x->trailingData(), $trailer ;
+
+        }
+
         {
             # Write
             # these tests come almost 100% from IO::String
@@ -600,7 +632,8 @@ EOM
 
             is myGZreadFile($name), "Heisan\nabcde\nf,g,h\nDEF.G.H" .
                                     ("1234567890" x 3) . "67890\n" .
-                                        "i(1)[1]\ni(2)[2]\ni(3)[3]\n\n";
+                                        "i(1)[1]\ni(2)[2]\ni(3)[3]\n\n",
+                                        "myGZreadFile ok";
 
 
         }
@@ -620,7 +653,9 @@ EOT
 
             my %opts = () ;
             my $iow = new $CompressClass $name, %opts;
+            is $iow->input_line_number, undef; 
             $iow->print($str) ;
+            is $iow->input_line_number, undef; 
             $iow->close ;
 
             my @tmp;
@@ -630,8 +665,8 @@ EOT
             
                 is $., 0; 
                 is $io->input_line_number, 0; 
-                ok ! $io->eof;
-                is $io->tell(), 0 ;
+                ok ! $io->eof, "eof";
+                is $io->tell(), 0, "tell 0" ;
                 #my @lines = <$io>;
                 my @lines = $io->getlines();
                 is @lines, 6
@@ -688,6 +723,29 @@ EOT
                 ok $lines[1] eq "and a single line.\n\n";
             }
             
+            {
+                # Record mode
+                my $reclen = 7 ;
+                my $expected_records = int(length($str) / $reclen)
+                                        + (length($str) % $reclen ? 1 : 0);
+                local $/ = \$reclen;
+
+                my $io = $UncompressClass->new($name);
+                is $., 0; 
+                is $io->input_line_number, 0; 
+
+                ok ! $io->eof;
+                my @lines = $io->getlines();
+                is $., $expected_records; 
+                is $io->input_line_number, $expected_records; 
+                ok $io->eof;
+                is @lines, $expected_records, 
+                    "Got $expected_records records\n" ;
+                ok $lines[0] eq substr($str, 0, $reclen)
+                    or print "# $lines[0]\n";
+                ok $lines[1] eq substr($str, $reclen, $reclen);
+            }
+
             {
                 local $/ = "is";
                 my $io = $UncompressClass->new($name);
@@ -725,11 +783,11 @@ EOT
 
                 is $io->read($buf, 0), 0, "Requested 0 bytes" ;
 
-                ok $io->read($buf, 3) == 3 ;
-                ok $buf eq "Thi";
+                is $io->read($buf, 3), 3 ;
+                is $buf, "Thi";
             
-                ok $io->sysread($buf, 3, 2) == 3 ;
-                ok $buf eq "Ths i"
+                is $io->sysread($buf, 3, 2), 3 ;
+                is $buf, "Ths i"
                     or print "# [$buf]\n" ;;
                 ok ! $io->eof;
             
@@ -823,6 +881,29 @@ EOT
                 ok $lines[1] eq "and a single line.\n\n";
             }
             
+            {
+                # Record mode
+                my $reclen = 7 ;
+                my $expected_records = int(length($str) / $reclen)
+                                        + (length($str) % $reclen ? 1 : 0);
+                local $/ = \$reclen;
+
+                my $io = $UncompressClass->new($name);
+                is $., 0; 
+                is $io->input_line_number, 0; 
+
+                ok ! $io->eof;
+                my @lines = $io->getlines();
+                is $., $expected_records; 
+                is $io->input_line_number, $expected_records; 
+                ok $io->eof;
+                is @lines, $expected_records, 
+                    "Got $expected_records records\n" ;
+                ok $lines[0] eq substr($str, 0, $reclen)
+                    or print "# $lines[0]\n";
+                ok $lines[1] eq substr($str, $reclen, $reclen);
+            }
+
             {
                 local $/ = "is";
                 my $io = $UncompressClass->new($name);
@@ -1110,51 +1191,65 @@ EOT
 
         foreach my $type (qw(buffer filename filehandle))
         {
-            title "$UncompressClass -- InputLength, read from $type";
-
-            my $compressed ; 
-            my $string = "some data";
-            my $c = new $CompressClass(\$compressed);
-            $c->write($string);
-            $c->close();
-
-            my $appended = "append";
-            my $comp_len = length $compressed;
-            $compressed .= $appended;
-
-            my $lex = new LexFile my $name ;
-            my $input ;
-            writeFile ($name, $compressed);
-
-            if ($type eq 'buffer')
+            foreach my $good (0, 1)
             {
-                $input = \$compressed;
-            }
-            if ($type eq 'filename')
-            {
-                $input = $name;
-            }
-            elsif ($type eq 'filehandle')
-            {
-                my $fh = new IO::File "<$name" ;
-                ok $fh, "opened file $name ok";
-                $input = $fh ;
-            }
+                title "$UncompressClass -- InputLength, read from $type, good data => $good";
 
-            my $x = new $UncompressClass($input, InputLength => $comp_len)  ;
-            ok $x, "  created $UncompressClass";
+                my $compressed ; 
+                my $string = "some data";
+                my $appended = "append";
 
-            my $len ;
-            my $output;
-            $len = $x->read($output, 100);
-            is $len, length($string);
-            is $output, $string;
+                if ($good)
+                {
+                    my $c = new $CompressClass(\$compressed);
+                    $c->write($string);
+                    $c->close();
+                }
+                else
+                {
+                    $compressed = $string ;
+                }
 
-            if ($type eq 'filehandle')
-            {
-                my $rest ;
-                $input->read($rest, 1000);
-                is $rest, $appended;
+                my $comp_len = length $compressed;
+                $compressed .= $appended;
+
+                my $lex = new LexFile my $name ;
+                my $input ;
+                writeFile ($name, $compressed);
+
+                if ($type eq 'buffer')
+                {
+                    $input = \$compressed;
+                }
+                if ($type eq 'filename')
+                {
+                    $input = $name;
+                }
+                elsif ($type eq 'filehandle')
+                {
+                    my $fh = new IO::File "<$name" ;
+                    ok $fh, "opened file $name ok";
+                    $input = $fh ;
+                }
+
+                my $x = new $UncompressClass($input, 
+                                             InputLength => $comp_len,
+                                             Transparent => 1)  ;
+                ok $x, "  created $UncompressClass";
+
+                my $len ;
+                my $output;
+                $len = $x->read($output, 100);
+
+                is $len, length($string);
+                is $output, $string;
+
+                if ($type eq 'filehandle')
+                {
+                    my $rest ;
+                    $input->read($rest, 1000);
+                    is $rest, $appended;
+                }
             }
 
 
@@ -1260,6 +1355,8 @@ EOT
                 is $u->read($buff, 1), 0;
                 ok $u->eof() ;
 
+                # getc returns undef on eof
+                is $u->getc(), undef;
                 $u->close();
 
             }
